library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity niosII_finalProject is
        port
        (
		  
					--VGA
					VGA_R : out std_logic_vector (9 downto 0);
					VGA_B : out std_logic_vector (9 downto 0);
					VGA_G : out std_logic_vector (9 downto 0);
					VGA_HS : out std_logic;
					VGA_VS : out std_logic;
					VGA_CLK : out std_logic;
					VGA_SYNC : out std_logic; 
					VGA_BLANK : out std_logic;
					
					--SRAM
					SRAM_ADDR : out std_logic_vector (17 downto 0);
					SRAM_DQ : inout std_logic_vector (15 downto 0);
					SRAM_WE_N : out std_logic;
					SRAM_OE_N : out std_logic;
					SRAM_UB_N : out std_logic;
					SRAM_LB_N : out std_logic;
					SRAM_CE_N : out std_logic;
					 
                -- LEDS
                LEDG    : out std_logic_vector(7 downto 0);
                

                -- DRAM ports
                DRAM_CLK, DRAM_CKE : OUT STD_LOGIC;
                DRAM_ADDR : OUT STD_LOGIC_VECTOR(11 DOWNTO 0);
                DRAM_BA_1, DRAM_BA_0 : BUFFER STD_LOGIC;
                DRAM_CS_N, DRAM_CAS_N, DRAM_RAS_N, DRAM_WE_N : OUT STD_LOGIC;
                DRAM_DQ : INOUT STD_LOGIC_VECTOR(15 DOWNTO 0);
                DRAM_UDQM, DRAM_LDQM : BUFFER STD_LOGIC ;

                -- CLOCK port
                CLOCK_50        :       in  std_logic;
					 CLOCK_27        :       in  std_logic;
					 
                -- RESET key
                SW         :       in  std_logic_vector(17 downto 0);
					 
					 GPIO_0				: inout std_logic_vector( 35 downto 0 );
					 
					 
					 --CFI
					 FL_ADDR : out std_logic_vector (21 downto 0);
					 FL_CE_N : out std_logic;
					 FL_OE_N : out std_logic;
					 FL_DQ : inout std_logic_vector (7 downto 0);
					 FL_RST_N : out std_logic;
					 FL_WE_N : out std_logic;
					 
					 UART_RXD : in std_logic
                );
end niosII_finalProject;


-- Library Clause(s) (optional)
-- Use Clause(s) (optional)

architecture structure of niosII_finalProject is

-- A component declaration declares the interface of an entity or
-- a design unit written in another language.  VHDL requires that
-- you declare a component if you do not intend to instantiate
-- an entity directly.  The component need not declare all the
-- generics and ports in the entity.  It may omit generics/ports
-- with default values.

component niosII is 
           port (
                 -- 1) global signals:
                    signal altpll_0_c0_out : OUT STD_LOGIC;
                    signal altpll_0_c1_out : OUT STD_LOGIC;
                    signal clk_0 : IN STD_LOGIC;
                    signal clk_1 : IN STD_LOGIC;
                    signal reset_n : IN STD_LOGIC;
                    signal sramConnector_0_inter_data : INOUT STD_LOGIC_VECTOR (15 DOWNTO 0);
                    signal vga_pll_c0_out : OUT STD_LOGIC;
						  
						  -- the_Controller1UART
                    signal rxd_to_the_Controller1UART : IN STD_LOGIC;
                    signal txd_from_the_Controller1UART : OUT STD_LOGIC;

                 -- the_Controller2UART
                    signal rxd_to_the_Controller2UART : IN STD_LOGIC;
                    signal txd_from_the_Controller2UART : OUT STD_LOGIC;

                 -- the_altpll_0
                    signal locked_from_the_altpll_0 : OUT STD_LOGIC;
                    signal phasedone_from_the_altpll_0 : OUT STD_LOGIC;

                 -- the_frameReader_0
                    signal coe_blue_export_from_the_frameReader_0 : OUT STD_LOGIC_VECTOR (9 DOWNTO 0);
                    signal coe_green_export_from_the_frameReader_0 : OUT STD_LOGIC_VECTOR (9 DOWNTO 0);
                    signal coe_red_export_from_the_frameReader_0 : OUT STD_LOGIC_VECTOR (9 DOWNTO 0);
                    signal coe_resetCol_export_to_the_frameReader_0 : IN STD_LOGIC;
                    signal coe_rowClock_export_to_the_frameReader_0 : IN STD_LOGIC;
                    signal coe_rst_export_from_the_frameReader_0 : OUT STD_LOGIC;
                    signal coe_x_export_to_the_frameReader_0 : IN STD_LOGIC_VECTOR (9 DOWNTO 0);
                    signal coe_y_export_to_the_frameReader_0 : IN STD_LOGIC_VECTOR (9 DOWNTO 0);

                 -- the_green_leds
                    signal out_port_from_the_green_leds : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
						  
						  -- the_player_select
						  signal in_port_to_the_player_select : IN STD_LOGIC_VECTOR (7 DOWNTO 0);

                 -- the_sdram_0
                    signal zs_addr_from_the_sdram_0 : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
                    signal zs_ba_from_the_sdram_0 : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
                    signal zs_cas_n_from_the_sdram_0 : OUT STD_LOGIC;
                    signal zs_cke_from_the_sdram_0 : OUT STD_LOGIC;
                    signal zs_cs_n_from_the_sdram_0 : OUT STD_LOGIC;
                    signal zs_dq_to_and_from_the_sdram_0 : INOUT STD_LOGIC_VECTOR (15 DOWNTO 0);
                    signal zs_dqm_from_the_sdram_0 : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
                    signal zs_ras_n_from_the_sdram_0 : OUT STD_LOGIC;
                    signal zs_we_n_from_the_sdram_0 : OUT STD_LOGIC;

                 -- the_sramConnector_0
                    signal coe_address_export_from_the_sramConnector_0 : OUT STD_LOGIC_VECTOR (17 DOWNTO 0);
                    signal coe_chipselect_export_from_the_sramConnector_0 : OUT STD_LOGIC;
                    signal coe_lowerbyte_export_from_the_sramConnector_0 : OUT STD_LOGIC;
                    signal coe_outputenable_export_from_the_sramConnector_0 : OUT STD_LOGIC;
                    signal coe_upperbyte_export_from_the_sramConnector_0 : OUT STD_LOGIC;
                    signal coe_writeenable_export_from_the_sramConnector_0 : OUT STD_LOGIC;
						  
						  -- the_tri_state_bridge_1_avalon_slave
                    signal address_to_the_cfi_flash_0 : OUT STD_LOGIC_VECTOR (21 DOWNTO 0);
                    signal data_to_and_from_the_cfi_flash_0 : INOUT STD_LOGIC_VECTOR (7 DOWNTO 0);
                    signal read_n_to_the_cfi_flash_0 : OUT STD_LOGIC;
                    signal select_n_to_the_cfi_flash_0 : OUT STD_LOGIC;
                    signal write_n_to_the_cfi_flash_0 : OUT STD_LOGIC;

                 -- the_vga_pll
                    signal locked_from_the_vga_pll : OUT STD_LOGIC;
                    signal phasedone_from_the_vga_pll : OUT STD_LOGIC;

                 -- the_vga_sync_0
                    signal coe_BLANK_export_from_the_vga_sync_0 : OUT STD_LOGIC;
                    signal coe_HSYNC_export_from_the_vga_sync_0 : OUT STD_LOGIC;
                    signal coe_SYNC_export_from_the_vga_sync_0 : OUT STD_LOGIC;
                    signal coe_VGAB_export_from_the_vga_sync_0 : OUT STD_LOGIC_VECTOR (9 DOWNTO 0);
                    signal coe_VGAG_export_from_the_vga_sync_0 : OUT STD_LOGIC_VECTOR (9 DOWNTO 0);
                    signal coe_VGAR_export_from_the_vga_sync_0 : OUT STD_LOGIC_VECTOR (9 DOWNTO 0);
                    signal coe_VSYNC_export_from_the_vga_sync_0 : OUT STD_LOGIC;
                    signal coe_colRST_export_from_the_vga_sync_0 : OUT STD_LOGIC;
                    signal coe_iBlue_export_to_the_vga_sync_0 : IN STD_LOGIC_VECTOR (9 DOWNTO 0);
                    signal coe_iCLK_export_to_the_vga_sync_0 : IN STD_LOGIC;
                    signal coe_iGreen_export_to_the_vga_sync_0 : IN STD_LOGIC_VECTOR (9 DOWNTO 0);
                    signal coe_iRST_export_to_the_vga_sync_0 : IN STD_LOGIC;
                    signal coe_iRed_export_to_the_vga_sync_0 : IN STD_LOGIC_VECTOR (9 DOWNTO 0);
                    signal coe_px_export_from_the_vga_sync_0 : OUT STD_LOGIC_VECTOR (9 DOWNTO 0);
                    signal coe_py_export_from_the_vga_sync_0 : OUT STD_LOGIC_VECTOR (9 DOWNTO 0);
                    signal coe_video_export_from_the_vga_sync_0 : OUT STD_LOGIC
                 );
end component niosII;

        signal BA : STD_LOGIC_VECTOR (1 downto 0);
        signal DQM : STD_LOGIC_VECTOR (1 downto 0);

        signal pll_c1:STD_LOGIC;
        signal pll_locked:STD_LOGIC;	  
        signal pll_phase: STD_LOGIC;
		  
		  signal vga_locked:STD_LOGIC;	  
        signal vga_phase: STD_LOGIC;
		  
		  signal vgain_clk : STD_LOGIC; 
		  
		  signal colorSplit :STD_LOGIC_VECTOR (23 DOWNTO 0);
		 
			signal px : STD_LOGIC_VECTOR (9 downto 0);
		  signal py : STD_LOGIC_VECTOR (9 downto 0);
		  
		  signal iRed : STD_LOGIC_VECTOR (9 downto 0) := "0000000000";
		  signal iBlue : STD_LOGIC_VECTOR (9 downto 0) := "0000000000"; 
		  signal iGreen : STD_LOGIC_VECTOR (9 downto 0) := "1111111111"; 
		  signal vgaRST : STD_LOGIC;
		  
		  signal rowClock : STD_LOGIC; 
		  
		  signal VS : STD_LOGIC; 
		  
		  signal CR : STD_LOGIC; 
		  
		  signal DummySerialTX :std_logic; 
		  
		  signal dummyReset : std_logic;

begin
			-- I have no idea why the SOPC-created IP does not turn the LCD on at all.
        -- I've explicitly turned it on with the next loc.      Note that LCD_ON is the
        -- VCC power line not the programmaticly controlled version that is bit banged.

		  VGA_CLK <= vgain_clk; 
		  
		  dummyReset <= '1';
		  
        DRAM_BA_1 <= BA(1);
        DRAM_BA_0 <= BA(0);
        DRAM_UDQM <= DQM(1);
        DRAM_LDQM <= DQM(0);
		  
		  --flash always on
		  FL_RST_N  <= '1';

        -- Component Instantiation Statement (optional)
        NIOS : niosII port map (
		  
		  --DRAM CLOCK
		  DRAM_CLK, pll_c1,
		  
		  --CLOCK AND RESET
		  CLOCK_50, CLOCK_27, dummyReset,
		  
		  --SRAM DATA LINES
		  SRAM_DQ, 
		  
		  --VGA CLOCK
		  vgain_clk,
		  
		  -- UARTs
		  GPIO_0(34), DummySerialTX, --Controller 1
		  UART_RXD, DummySerialTX, --Controller 2
			
			--PLL SIGNALS
		   pll_locked, pll_phase,
			
			--frameReader
			iBlue,iGreen,iRed,CR,rowClock,vgaRST,px,py,
			
			--GREEN LEDS
        LEDG,
		  
		  SW(7 downto 0),
		  
			--SDRAM STUFF
		  DRAM_ADDR, BA, DRAM_CAS_N, DRAM_CKE, DRAM_CS_N,

        DRAM_DQ, DQM, DRAM_RAS_N, DRAM_WE_N,
		  
		  --SRAM STUFF
		  SRAM_ADDR, SRAM_CE_N, SRAM_LB_N, SRAM_OE_N, SRAM_UB_N, SRAM_WE_N,
		  
		  --CFI
		  FL_ADDR, FL_DQ,FL_OE_N,FL_CE_N,FL_WE_N,
		  
		  --PLL FOR THE VGA 
		  vga_locked,vga_phase,
		  
		  --VGA stuff
		  VGA_BLANK, VGA_HS, VGA_SYNC, VGA_B, VGA_G, VGA_R, VGA_VS,CR, --sync and color
		  iBlue,vgain_clk , iGreen, vgaRST, iRed, --inputs
        px, py, rowClock --pixel locations
		  );
		  
end structure;
		  
